@@ -168,10 +168,10 @@ ukb_icd_prevalence <- function(data, icd.code, icd.version = 10) {
 #' @param data A UKB dataset (or subset) created with \code{\link{ukb_df}}.
 #' @param icd.code ICD disease code(s) e.g. "I74". Use a regular expression to specify a broader set of diagnoses, e.g. "I" captures all Diseases of the circulatory system, I00-I99, "C|D[0-4]." captures all Neoplasms, C00-D49. Default is the WHO top 3 causes of death globally in 2015, see \url{http://www.who.int/healthinfo/global_burden_disease/GlobalCOD_method_2000_2015.pdf?ua=1}.
 #' @param reference.var UKB ICD frequencies will be calculated by levels of this variable. If continuous, by default it is cut into 10 intervals of approximately equal size (set with n.groups).
-#' @param n.groups Number of approximately equal-sized groups to split a continous variable into.
+#' @param n.groups Number of approximately equal-sized groups to split a continuous variable into.
 #' @param icd.version The ICD version (or revision) number, 9 or 10.
 #' @param freq.plot If TRUE returns a plot of ICD diagnosis by target variable. If FALSE (default) returns a dataframe.
-#' @param legend.col Number of columns for the legend. (Defeault = 1).
+#' @param legend.col Number of columns for the legend. (Default = 1).
 #' @param legend.pos Legend position, default = "right".
 #' @param icd.labels Character vector of ICD labels for the plot legend. Default = V1 to VN.
 #' @param plot.title Title for the plot. Default describes the default icd.codes, WHO top 6 cause of death 2015.
@@ -219,7 +219,7 @@ ukb_icd_freq_by <- function(
   rm(l)
 
   if(is.numeric(df[[reference.var]])) {
-    o <- order(as.numeric(gsub("[[:punct:]]", "", gsub(",.*$", "", levels(x$group)))))
+    o <- order(as.numeric(gsub("[\\(\\[]", "", gsub(",.*$", "", levels(x$group)))))
     levels(x$group)[o]
     x$group <- factor(x$group, levels = levels(x$group)[o])
   }
